@@ -7,7 +7,8 @@ the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "http" or "web"
 or "html" (this is the default).
 
-Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2010 Davide Alberani <da@erlug.linux.it>
+               2008 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,28 +26,50 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import sys
+import logging
 from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
 from imdb import IMDbBase, imdbURL_movie_main, imdbURL_person_main, \
-                imdbURL_character_main, imdbURL_company_main, imdbURL_find
+                imdbURL_character_main, imdbURL_company_main, \
+                imdbURL_keyword_main, imdbURL_find, imdbURL_top250, \
+                imdbURL_bottom100
+from imdb.utils import analyze_title
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 
 import searchMovieParser
 import searchPersonParser
 import searchCharacterParser
 import searchCompanyParser
+import searchKeywordParser
 import movieParser
 import personParser
 import characterParser
 import companyParser
+import topBottomParser
+
+# Logger for miscellaneous functions.
+_aux_logger = logging.getLogger('imdbpy.parser.http.aux')
+
+IN_GAE = False
+try:
+    import google.appengine
+    IN_GAE = True
+    _aux_logger.info('IMDbPY is running in the Google App Engine environment')
+except ImportError:
+    pass
 
 
 class _ModuleProxy:
     """A proxy to instantiate and access parsers."""
-    def __init__(self, module, defaultKeys=None):
+    def __init__(self, module, defaultKeys=None, oldParsers=False,
+                useModule=None, fallBackToNew=False):
         """Initialize a proxy for the given module; defaultKeys, if set,
         muste be a dictionary of values to set for instanced objects."""
+        if oldParsers or fallBackToNew:
+            _aux_logger.warn('The old set of parsers was removed; falling ' \
+                            'back to the new parsers.')
+        self.useModule = useModule
         if defaultKeys is None:
             defaultKeys = {}
         self._defaultKeys = defaultKeys
@@ -59,7 +82,11 @@ class _ModuleProxy:
         if name in _sm._OBJECTS:
             _entry = _sm._OBJECTS[name]
             # Initialize the parser.
-            obj = _entry[0]()
+            kwds = {}
+            if self.useModule:
+                kwds = {'useModule': self.useModule}
+            parserClass = _entry[0][0]
+            obj = parserClass(**kwds)
             attrsToSet = self._defaultKeys.copy()
             attrsToSet.update(_entry[1] or {})
             # Set attribute to the object.
@@ -83,15 +110,32 @@ _cookie_id = 'rH1jNAkjTlNXvHolvBVBsgaPICNZbNdjVjzFwzas9JRmusdjVoqBs/Hs12NR+1WFxE
 _cookie_uu = 'su4/m8cho4c6HP+W1qgq6wchOmhnF0w+lIWvHjRUPJ6nRA9sccEafjGADJ6hQGrMd4GKqLcz2X4z5+w+M4OIKnRn7FpENH7dxDQu3bQEHyx0ZEyeRFTPHfQEX03XF+yeN1dsPpcXaqjUZAw+lGRfXRQEfz3RIX9IgVEffdBAHw2wQXyf9xdMPrQELw0QNB8dsffsqcdQemjPB0w+moLcPh0JrKrHJ9hjBzdMPpcXTH7XRwwOk='
 
 
+class _FakeURLOpener(object):
+    """Fake URLOpener object, used to return empty strings instead of
+    errors.
+    """
+    def __init__(self, url, headers):
+        self.url = url
+        self.headers = headers
+    def read(self, *args, **kwds): return ''
+    def close(self, *args, **kwds): pass
+    def info(self, *args, **kwds): return self.headers
+
+
 class IMDbURLopener(FancyURLopener):
     """Fetch web pages and handle errors."""
+    _logger = logging.getLogger('imdbpy.parser.http.urlopener')
+
     def __init__(self, *args, **kwargs):
+        self._last_url = u''
         FancyURLopener.__init__(self, *args, **kwargs)
         # Headers to add to every request.
         # XXX: IMDb's web server doesn't like urllib-based programs,
         #      so lets fake to be Mozilla.
         #      Wow!  I'm shocked by my total lack of ethic! <g>
-        self.set_header('User-agent', 'Mozilla/5.0')
+        for header in ('User-Agent', 'User-agent', 'user-agent'):
+            self.del_header(header)
+        self.set_header('User-Agent', 'Mozilla/5.0')
         # XXX: This class is used also to perform "Exact Primary
         #      [Title|Name]" searches, and so by default the cookie is set.
         c_header = 'id=%s; uu=%s' % (_cookie_id, _cookie_uu)
@@ -134,9 +178,10 @@ class IMDbURLopener(FancyURLopener):
                 self.set_header('Range', 'bytes=0-%d' % size)
             uopener = self.open(url)
             kwds = {}
-            if PY_VERSION > (2, 3):
+            if PY_VERSION > (2, 3) and not IN_GAE:
                 kwds['size'] = size
             content = uopener.read(**kwds)
+            self._last_url = uopener.url
             # Maybe the server is so nice to tell us the charset...
             server_encode = uopener.info().getparam('charset')
             # Otherwise, look at the content-type HTML meta tag.
@@ -170,12 +215,16 @@ class IMDbURLopener(FancyURLopener):
         if encode is None:
             encode = 'latin_1'
             # The detection of the encoding is error prone...
-            import warnings
-            warnings.warn('Unable to detect the encoding of the retrieved '
-                        'page [%s]; falling back to default latin1.' % encode)
+            self._logger.warn('Unable to detect the encoding of the retrieved '
+                        'page [%s]; falling back to default latin1.', encode)
+        ##print unicode(content, encode, 'replace').encode('utf8')
         return unicode(content, encode, 'replace')
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
+        if errcode == 404:
+            self._logger.warn('404 code returned for %s: %s (headers: %s)',
+                                url, errmsg, headers)
+            return _FakeURLOpener(url, headers)
         raise IMDbDataAccessError, {'url': 'http:%s' % url,
                                     'errcode': errcode,
                                     'errmsg': errmsg,
@@ -200,9 +249,11 @@ class IMDbHTTPAccessSystem(IMDbBase):
     """The class used to access IMDb's data through the web."""
 
     accessSystem = 'http'
+    _http_logger = logging.getLogger('imdbpy.parser.http')
 
-    def __init__(self, isThin=0, adultSearch=1, proxy=-1,
-                cookie_id=-1, cookie_uu=None, *arguments, **keywords):
+    def __init__(self, isThin=0, adultSearch=1, proxy=-1, oldParsers=False,
+                fallBackToNew=False, useModule=None, cookie_id=-1,
+                cookie_uu=None, *arguments, **keywords):
         """Initialize the access system."""
         IMDbBase.__init__(self, *arguments, **keywords)
         self.urlOpener =  IMDbURLopener()
@@ -228,16 +279,41 @@ class IMDbHTTPAccessSystem(IMDbBase):
                 self.set_cookies(cookie_id, cookie_uu)
         if proxy != -1:
             self.set_proxy(proxy)
+        if useModule is not None:
+            if not isinstance(useModule, (list, tuple)) and ',' in useModule:
+                useModule = useModule.split(',')
         _def = {'_modFunct': self._defModFunct, '_as': self.accessSystem}
         # Proxy objects.
-        self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def)
-        self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def)
-        self.scProxy = _ModuleProxy(searchCharacterParser, defaultKeys=_def)
-        self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def)
-        self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def)
-        self.pProxy = _ModuleProxy(personParser, defaultKeys=_def)
-        self.cProxy = _ModuleProxy(characterParser, defaultKeys=_def)
-        self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def)
+        self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.scProxy = _ModuleProxy(searchCharacterParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.skProxy = _ModuleProxy(searchKeywordParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.pProxy = _ModuleProxy(personParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.cProxy = _ModuleProxy(characterParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.topBottomProxy = _ModuleProxy(topBottomParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -331,6 +407,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _retrieve(self, url, size=-1):
         """Retrieve the given URL."""
+        ##print url
+        self._http_logger.debug('fetching url %s (size: %d)', url, size)
         return self.urlOpener.retrieve_unicode(url, size=size)
 
     def _get_search_content(self, kind, ton, results):
@@ -343,13 +421,16 @@ class IMDbHTTPAccessSystem(IMDbBase):
             ton = ton.encode('utf-8')
         ##params = 'q=%s&%s=on&mx=%s' % (quote_plus(ton), kind, str(results))
         params = 's=%s;mx=%s;q=%s' % (kind, str(results), quote_plus(ton))
+        if kind == 'ep':
+            params = params.replace('s=ep;', 's=tt;ttype=ep;', 1)
         cont = self._retrieve(imdbURL_find % params)
         #print 'URL:', imdbURL_find % params
-        if cont.find('more than 500 partial matches') == -1:
+        if cont.find('Your search returned more than') == -1 or \
+                cont.find("displayed the exact matches") == -1:
             return cont
         # The retrieved page contains no results, because too many
         # titles or names contain the string we're looking for.
-        params = 'q=%s;more=%s' % (quote_plus(ton), kind)
+        params = 's=%s;q=%s;lm=0' % (kind, quote_plus(ton))
         size = 22528 + results * 512
         return self._retrieve(imdbURL_find % params, size=size)
 
@@ -361,8 +442,14 @@ class IMDbHTTPAccessSystem(IMDbBase):
         ##params = 'q=%s&tt=on&mx=%s' % (quote_plus(title), str(results))
         ##cont = self._retrieve(imdbURL_find % params)
         cont = self._get_search_content('tt', title, results)
-        return self.smProxy.search_movie_parser.parse(cont,
-                                                    results=results)['data']
+        return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
+
+    def _search_episode(self, title, results):
+        t_dict = analyze_title(title)
+        if t_dict['kind'] == 'episode':
+            title = t_dict['title']
+        cont = self._get_search_content('ep', title, results)
+        return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
 
     def get_movie_main(self, movieID):
         if not self.isThin:
@@ -411,7 +498,10 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_release_dates(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'releaseinfo')
-        return self.mProxy.releasedates_parser.parse(cont)
+        ret = self.mProxy.releasedates_parser.parse(cont)
+        ret['info sets'] = ('release dates', 'akas')
+        return ret
+    get_movie_akas = get_movie_release_dates
 
     def get_movie_vote_details(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'ratings')
@@ -491,7 +581,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_guests(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'epcast')
-        return self.mProxy.episodes_parser.parse(cont)
+        return self.mProxy.episodes_cast_parser.parse(cont)
     get_movie_episodes_cast = get_movie_guests
 
     def get_movie_merchandising_links(self, movieID):
@@ -531,6 +621,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'tvschedule')
         return self.mProxy.airing_parser.parse(cont)
 
+    get_movie_tv_schedule = get_movie_airing
+
     def get_movie_synopsis(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'synopsis')
         return self.mProxy.synopsis_parser.parse(cont)
@@ -547,8 +639,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
         #params = 'q=%s&nm=on&mx=%s' % (quote_plus(name), str(results))
         #cont = self._retrieve(imdbURL_find % params)
         cont = self._get_search_content('nm', name, results)
-        return self.spProxy.search_person_parser.parse(cont,
-                                                    results=results)['data']
+        return self.spProxy.search_person_parser.parse(cont, results=results)['data']
 
     def get_person_main(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'maindetails')
@@ -571,9 +662,9 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person_main % personID + 'otherworks')
         return self.pProxy.otherworks_parser.parse(cont, getRefs=self._getRefs)
 
-    def get_person_agent(self, personID):
-        cont = self._retrieve(imdbURL_person_main % personID + 'agent')
-        return self.pProxy.agent_parser.parse(cont)
+    #def get_person_agent(self, personID):
+    #    cont = self._retrieve(imdbURL_person_main % personID + 'agent')
+    #    return self.pProxy.agent_parser.parse(cont)
 
     def get_person_publicity(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'publicity')
@@ -605,8 +696,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _search_character(self, name, results):
         cont = self._get_search_content('char', name, results)
-        return self.scProxy.search_character_parser.parse(cont,
-                                                    results=results)['data']
+        return self.scProxy.search_character_parser.parse(cont, results=results)['data']
 
     def get_character_main(self, characterID):
         cont = self._retrieve(imdbURL_character_main % characterID)
@@ -633,7 +723,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _search_company(self, name, results):
         cont = self._get_search_content('co', name, results)
-        return self.scompProxy.search_company_parser.parse(cont,
+        url = self.urlOpener._last_url
+        return self.scompProxy.search_company_parser.parse(cont, url=url,
                                                     results=results)['data']
 
     def get_company_main(self, companyID):
@@ -641,4 +732,40 @@ class IMDbHTTPAccessSystem(IMDbBase):
         ret = self.compProxy.company_main_parser.parse(cont)
         return ret
 
+    def _search_keyword(self, keyword, results):
+        # XXX: the IMDb web server seems to have some serious problem with
+        #      non-ascii keyword.
+        #      E.g.: http://akas.imdb.com/keyword/fianc%E9/
+        #      will return a 500 Internal Server Error: Redirect Recursion.
+        keyword = keyword.encode('utf8', 'ignore')
+        try:
+            cont = self._get_search_content('kw', keyword, results)
+        except IMDbDataAccessError:
+            self._http_logger.warn('unable to search for keyword %s', keyword,
+                                    exc_info=True)
+            return []
+        return self.skProxy.search_keyword_parser.parse(cont, results=results)['data']
+
+    def _get_keyword(self, keyword, results):
+        keyword = keyword.encode('utf8', 'ignore')
+        try:
+            cont = self._retrieve(imdbURL_keyword_main % keyword)
+        except IMDbDataAccessError:
+            self._http_logger.warn('unable to get keyword %s', keyword,
+                                    exc_info=True)
+            return []
+        return self.skProxy.search_moviekeyword_parser.parse(cont, results=results)['data']
+
+    def _get_top_bottom_movies(self, kind):
+        if kind == 'top':
+            parser = self.topBottomProxy.top250_parser
+            url = imdbURL_top250
+        elif kind == 'bottom':
+            parser = self.topBottomProxy.bottom100_parser
+            url = imdbURL_bottom100
+        else:
+            return []
+        cont = self._retrieve(url)
+        return parser.parse(cont)['data']
+
 
