@@ -1,11 +1,12 @@
-#!/usr/bin/env python
+#!/usr/local/bin/python2.6
 """
 get_character.py
 
 Usage: get_character "characterID"
 
-Show some info about the character with the given imdbID (e.g. '0000001'
-for "Jesse James".
+Show some info about the character with the given characterID (e.g. '0000001'
+for "Jesse James", using 'http' or 'mobile').
+Notice that characterID, using 'sql', are not the same IDs used on the web.
 """
 
 import sys
@@ -20,10 +21,10 @@ except ImportError:
 
 if len(sys.argv) != 2:
     print 'Only one argument is required:'
-    print '  %s "imdbID"' % sys.argv[0]
+    print '  %s "characterID"' % sys.argv[0]
     sys.exit(2)
 
-imdbID = sys.argv[1]
+characterID = sys.argv[1]
 
 i = imdb.IMDb()
 
@@ -31,8 +32,8 @@ out_encoding = sys.stdout.encoding or sys.getdefaultencoding()
 
 try:
     # Get a character object with the data about the character identified by
-    # the given imdbID.
-    character = i.get_character(imdbID)
+    # the given characterID.
+    character = i.get_character(characterID)
 except imdb.IMDbError, e:
     print "Probably you're not connected to Internet.  Complete error report:"
     print e
@@ -40,7 +41,7 @@ except imdb.IMDbError, e:
 
 
 if not character:
-    print 'It seems that there\'s no character with imdbID "%s"' % imdbID
+    print 'It seems that there\'s no character with characterID "%s"' % characterID
     sys.exit(4)
 
 # XXX: this is the easier way to print the main info about a character;
