@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/local/bin/python2.6
 """
 search_movie.py
 
@@ -47,8 +47,8 @@ print 'movieID\t: imdbID : title'
 
 # Print the long imdb title for every movie.
 for movie in results:
-    print '%s\t: %s : %s' % (movie.movieID,
-                    i.get_imdbID(movie),
-                    movie['long imdb title'].encode(out_encoding, 'replace'))
+    outp = u'%s\t: %s : %s' % (movie.movieID, i.get_imdbID(movie),
+                                movie['long imdb title'])
+    print outp.encode(out_encoding, 'replace')
 
 
